@@ -309,7 +309,7 @@ impl<'a> Tokenizer<'a> {
             location.column = 1;
         }
         debug_assert!(position <= target);
-        location.column += target - position;
+        location.column += (target - position) as u32;
         self.last_known_source_location.set((SourcePosition(target), location));
         location
     }
@@ -383,10 +383,10 @@ pub struct SourcePosition(usize);
 #[derive(PartialEq, Eq, Debug, Clone, Copy)]
 pub struct SourceLocation {
     /// The line number, starting at 1 for the first line.
-    pub line: usize,
+    pub line: u32,
 
     /// The column number within a line, starting at 1 for first the character of the line.
-    pub column: usize,
+    pub column: u32,
 }
 
 
@@ -859,7 +859,7 @@ fn consume_numeric<'a>(tokenizer: &mut Tokenizer<'a>) -> Token<'a> {
     if !tokenizer.is_eof() && tokenizer.next_byte_unchecked() == b'%' {
         tokenizer.advance(1);
         return Percentage(PercentageValue {
-            unit_value: value as f32 / 100.,
+            unit_value: (value / 100.) as f32,
             int_value: int_value,
             has_sign: has_sign,
         })
